# -*- coding: utf-8 -*-
"""
youtube.py is really lite python liberary with fast downloader
"""

__title__ = "youtube.py"
__version__ = "1.1.6"
__author__ = "Mayank Gupta"
__license__ = "MIT License"
__copyright__ = "Copyright 2020 YouTube.py (Mayank Gupta)"
__github__ = "https://github.com/youtube-py/youtube.py"
__issues__ = "https://github.com/youtube-py/youtube.py/issues"
__docs__ = "https://youtube-python.mayankfawkes.xyz/"

